Exceptions
==========

.. py:currentmodule:: pgpy.errors


:py:class:`PGPError`
--------------------

.. autoexception:: PGPError


:py:class:`PGPEncryptionError`
------------------------------

.. autoexception:: PGPEncryptionError

:py:class:`PGPDecryptionError`
------------------------------

.. autoexception:: PGPDecryptionError

:py:class:`PGPOpenSSLCipherNotSupported`
----------------------------------------

.. autoexception:: PGPOpenSSLCipherNotSupported

:py:class:`PGPInsecureCipher`
-----------------------------

.. autoexception:: PGPInsecureCipher

:py:class:`WontImplementError`
------------------------------

.. autoexception:: WontImplementError
